@@ -10,8 +10,8 @@ from collections import OrderedDict
 
 import numpy as np
 
-from astropy.nddata import NDDataArray
-from astropy.nddata.nduncertainty import StdDevUncertainty, NDUncertainty, MissingDataAssociationException
+from astropy.nddata import (NDDataArray, StdDevUncertainty, NDUncertainty,
+                            MissingDataAssociationException)
 from astropy.io import fits, registry
 from astropy import units as u
 from astropy import log
@@ -22,7 +22,10 @@ from astropy.wcs import WCS
 # described in https://github.com/astropy/astropy/issues/4825
 import astropy
 from distutils.version import LooseVersion
-if LooseVersion(astropy.__version__) < LooseVersion('1.2'):  # pragma: no cover
+
+_ASTROPY_LT_1_2 = LooseVersion(astropy.__version__) < LooseVersion('1.2')
+
+if _ASTROPY_LT_1_2:
 
     class ParentNDDataDescriptor(object):
         def __get__(self, obj, objtype=None):
@@ -59,6 +62,49 @@ if LooseVersion(astropy.__version__) < LooseVersion('1.2'):  # pragma: no cover
 __all__ = ['CCDData', 'fits_ccddata_reader', 'fits_ccddata_writer']
 
 
+# Global value which can turn on/off the unit requirements when creating a
+# CCDData. Should be used with care because several functions actually break
+# if the unit is None!
+_config_ccd_requires_unit = True
+
+if not _ASTROPY_LT_1_2:
+    from astropy.utils.decorators import sharedmethod
+
+    def _arithmetic(op):
+        """Decorator factory which temporarly disables the need for a unit when
+        creating a new CCDData instance. The final result must have a unit.
+
+        Parameters
+        ----------
+        op : function
+            The function to apply. Supported are:
+
+            - ``np.add``
+            - ``np.subtract``
+            - ``np.multiply``
+            - ``np.true_divide``
+
+        Notes
+        -----
+        Should only be used on CCDData ``add``, ``subtract``, ``divide`` or
+        ``multiply`` because only these methods from NDArithmeticMixin are
+        overwritten.
+        """
+        def decorator(func):
+            def inner(self, operand, operand2=None, **kwargs):
+                global _config_ccd_requires_unit
+                _config_ccd_requires_unit = False
+                result = self._prepare_then_do_arithmetic(op, operand,
+                                                          operand2, **kwargs)
+                # Wrap it again as CCDData so it checks the final unit.
+                _config_ccd_requires_unit = True
+                return result.__class__(result)
+            inner.__doc__ = ("See `astropy.nddata.NDArithmeticMixin.{}`."
+                             "".format(func.__name__))
+            return sharedmethod(inner)
+        return decorator
+
+
 class CCDData(NDDataArray):
     """A class describing basic CCD data.
 
@@ -161,7 +207,10 @@ class CCDData(NDDataArray):
             raise ValueError("can't have both header and meta.")
 
         super(CCDData, self).__init__(*args, **kwd)
-        if self.unit is None:
+
+        # Check if a unit is set. This can be temporarly disabled by the
+        # _CCDDataUnit contextmanager.
+        if _config_ccd_requires_unit and self.unit is None:
             raise ValueError("a unit for CCDData must be specified.")
 
     @property
@@ -516,6 +565,15 @@ class CCDData(NDDataArray):
         return self._ccddata_arithmetic(other, np.subtract,
                                         scale_uncertainty=False)
 
+    # Use NDDataArithmetic methods if astropy version is 1.2 or greater
+    if not _ASTROPY_LT_1_2:
+        del add, subtract, divide, multiply, _ccddata_arithmetic
+
+        add = _arithmetic(np.add)(NDDataArray.add)
+        subtract = _arithmetic(np.subtract)(NDDataArray.subtract)
+        multiply = _arithmetic(np.multiply)(NDDataArray.multiply)
+        divide = _arithmetic(np.true_divide)(NDDataArray.divide)
+
     def _insert_in_metadata_fits_safe(self, key, value):
         """
         Insert key/value pair into metadata in a way that FITS can serialize.
@@ -550,7 +608,8 @@ class CCDData(NDDataArray):
             # problems...
             # Shorten, sort of...
             short_name = _short_names[key]
-            self.meta[key] = (short_name, "Shortened name for ccdproc command")
+            self.meta['HIERARCH {0}'.format(key.upper())] = (
+                short_name, "Shortened name for ccdproc command")
             self.meta[short_name] = value
         else:
             self.meta[key] = value
@@ -614,18 +673,18 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
     with fits.open(filename, **kwd) as hdus:
         hdr = hdus[hdu].header
 
-        if hdu_uncertainty in hdus:
+        if hdu_uncertainty is not None and hdu_uncertainty in hdus:
             uncertainty = StdDevUncertainty(hdus[hdu_uncertainty].data)
         else:
             uncertainty = None
 
-        if hdu_mask in hdus:
+        if hdu_mask is not None and hdu_mask in hdus:
             # Mask is saved as uint but we want it to be boolean.
             mask = hdus[hdu_mask].data.astype(np.bool_)
         else:
             mask = None
 
-        if hdu_flags in hdus:
+        if hdu_flags is not None and hdu_flags in hdus:
             raise NotImplementedError('loading flags is currently not '
                                       'supported.')
 
