@@ -6,9 +6,8 @@ from __future__ import (absolute_import, division, print_function,
 import numpy as np
 from numpy import ma
 from .ccddata import CCDData
-from .core import trim_image, sigma_func
+from .core import sigma_func
 
-from astropy.stats import median_absolute_deviation
 from astropy.nddata import StdDevUncertainty
 from astropy import log
 
@@ -32,7 +31,7 @@ class Combiner(object):
 
     dtype : str or `numpy.dtype` or None, optional
         Allows user to set dtype. See `numpy.array` ``dtype`` parameter
-        description.
+        description. If ``None`` it uses ``np.float64``.
         Default is ``None``.
 
     Raises
@@ -41,8 +40,8 @@ class Combiner(object):
         If the ``ccd_list`` are not `~ccdproc.CCDData` objects, have different
         units, or are different shapes.
 
-    Notes
-    -----
+    Examples
+    --------
     The following is an example of combining together different
     `~ccdproc.CCDData` objects::
 
@@ -94,7 +93,7 @@ class Combiner(object):
         self.weights = None
         self._dtype = dtype
 
-        #set up the data array
+        # set up the data array
         new_shape = (len(ccd_list),) + default_shape
         self.data_arr = ma.masked_all(new_shape, dtype=dtype)
 
@@ -175,7 +174,7 @@ class Combiner(object):
                                     "the same length as the number of images.")
             # reshape so that broadcasting occurs properly
             for i in range(len(self.data_arr.data.shape)-1):
-                self._scaling = self.scaling[:,np.newaxis]
+                self._scaling = self.scaling[:, np.newaxis]
 
     # set up IRAF-like minmax clipping
     def clip_extrema(self, nlow=0, nhigh=0):
@@ -185,23 +184,6 @@ class Combiner(object):
         image.  For example, the image will be a combination of
         Nimages-nlow-nhigh pixel values instead of the combination of Nimages.
 
-        Note that this differs slightly from the nominal IRAF behavior when
-        other masks are in use.  For example, if the nhigh>=1 and any
-        pixel is already masked for some other reason, then this algorithm will
-        count the masking of that pixel toward the count of nhigh masked pixels.
-        IRAF behaves slightly differently in this case (see IRAF help for that
-        behavior): http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?imcombine
-
-        Here is a copy of the relevant IRAF help text:
-
-        nlow = 1, nhigh = (minmax)
-            The number of low and high pixels to be rejected by the "minmax"
-            algorithm. These numbers are converted to fractions of the total
-            number of input images so that if no rejections have taken place
-            the specified number of pixels are rejected while if pixels have
-            been rejected by masking, thresholding, or nonoverlap, then the
-            fraction of the remaining pixels, truncated to an integer, is used.
-
         Parameters
         -----------
         nlow : int or None, optional
@@ -213,6 +195,29 @@ class Combiner(object):
             If not None, the number of high values to reject from the
             combination.
             Default is 0.
+
+        Notes
+        -----
+        Note that this differs slightly from the nominal IRAF imcombine
+        behavior when other masks are in use.  For example, if ``nhigh>=1`` and
+        any pixel is already masked for some other reason, then this algorithm
+        will count the masking of that pixel toward the count of nhigh masked
+        pixels.
+
+        Here is a copy of the relevant IRAF help text [0]_:
+
+        nlow = 1, nhigh = (minmax)
+            The number of low and high pixels to be rejected by the "minmax"
+            algorithm. These numbers are converted to fractions of the total
+            number of input images so that if no rejections have taken place
+            the specified number of pixels are rejected while if pixels have
+            been rejected by masking, thresholding, or nonoverlap, then the
+            fraction of the remaining pixels, truncated to an integer, is used.
+
+        References
+        ----------
+        .. [0] image.imcombine help text.
+           http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?imcombine
         """
 
         if nlow is None:
@@ -221,13 +226,14 @@ class Combiner(object):
             nhigh = 0
 
         argsorted = np.argsort(self.data_arr.data, axis=0)
-        mg = np.mgrid[[slice(ndim) for i, ndim in enumerate(self.data_arr.shape) if i > 0]]
+        mg = np.mgrid[[slice(ndim)
+                       for i, ndim in enumerate(self.data_arr.shape) if i > 0]]
         for i in range(-1*nhigh, nlow):
             # create a tuple with the indices
-            where = tuple([argsorted[i,:,:].ravel()] + [i.ravel() for i in mg])
+            where = tuple([argsorted[i, :, :].ravel()] +
+                          [i.ravel() for i in mg])
             self.data_arr.mask[where] = True
 
-
     # set up min/max clipping algorithms
     def minmax_clipping(self, min_clip=None, max_clip=None):
         """Mask all pixels that are below min_clip or above max_clip.
@@ -442,13 +448,14 @@ class Combiner(object):
         return combined_image
 
 
-def combine(img_list, output_file=None, method='average', weights=None,
-            scale=None, mem_limit=16e9,
+def combine(img_list, output_file=None,
+            method='average', weights=None, scale=None, mem_limit=16e9,
             clip_extrema=False, nlow=1, nhigh=1,
             minmax_clip=False, minmax_clip_min=None, minmax_clip_max=None,
             sigma_clip=False,
             sigma_clip_low_thresh=3, sigma_clip_high_thresh=3,
-            sigma_clip_func=ma.mean, sigma_clip_dev_func=ma.std, **ccdkwargs):
+            sigma_clip_func=ma.mean, sigma_clip_dev_func=ma.std,
+            dtype=None, combine_uncertainty_function=None, **ccdkwargs):
     """
     Convenience function for combining multiple images.
 
@@ -460,8 +467,8 @@ def combine(img_list, output_file=None, method='average', weights=None,
         ",".
 
     output_file : str or None, optional
-        Optional output fits file-name to which the final output can be directly
-        written.
+        Optional output fits file-name to which the final output can be
+        directly written.
         Default is ``None``.
 
     method : str, optional
@@ -492,10 +499,10 @@ def combine(img_list, output_file=None, method='average', weights=None,
     clip_extrema : bool, optional
         Set to True if you want to mask pixels using an IRAF-like minmax
         clipping algorithm.  The algorithm will mask the lowest nlow values and
-        the highest nhigh values before combining the values to make up a single
-        pixel in the resulting image.  For example, the image will be a
-        combination of Nimages-low-nhigh pixel values instead of the combination
-        of Nimages.
+        the highest nhigh values before combining the values to make up a
+        single pixel in the resulting image.  For example, the image will be a
+        combination of Nimages-low-nhigh pixel values instead of the
+        combination of Nimages.
 
         Parameters below are valid only when clip_extrema is set to True,
         see :meth:`Combiner.clip_extrema` for the parameter description:
@@ -534,6 +541,16 @@ def combine(img_list, output_file=None, method='average', weights=None,
         - ``sigma_clip_func`` : function, optional
         - ``sigma_clip_dev_func`` : function, optional
 
+    dtype : str or `numpy.dtype` or None, optional
+        The intermediate and resulting ``dtype`` for the combined CCDs. See
+        `ccdproc.Combiner`. If ``None`` this is set to ``float64``.
+        Default is ``None``.
+
+    combine_uncertainty_function : callable, None, optional
+        If ``None`` use the default uncertainty func when using average or
+        median combine, otherwise use the function provided.
+        Default is ``None``.
+
     ccdkwargs : Other keyword arguments for `ccdproc.fits_ccddata_reader`.
 
     Returns
@@ -565,20 +582,47 @@ def combine(img_list, output_file=None, method='average', weights=None,
         # User has provided fits filenames to read from
         ccd = CCDData.read(img_list[0], **ccdkwargs)
 
+    # If uncertainty_func is given for combine this will create an uncertainty
+    # even if the originals did not have one. In that case we need to create
+    # an empty placeholder.
+    if ccd.uncertainty is None and combine_uncertainty_function is not None:
+        ccd.uncertainty = StdDevUncertainty(np.zeros(ccd.data.shape))
+
+    if dtype is None:
+        dtype = np.float64
+
+    # Convert the master image to the appropriate dtype so when overwriting it
+    # later the data is not downcast and the memory consumption calculation
+    # uses the internally used dtype instead of the original dtype. #391
+    if ccd.data.dtype != dtype:
+        ccd.data = ccd.data.astype(dtype)
+
     size_of_an_img = ccd.data.nbytes
-    if ccd.uncertainty is not None:
-        size_of_an_img += ccd.uncertainty.nbytes
+    try:
+        size_of_an_img += ccd.uncertainty.array.nbytes
+    # In case uncertainty is None it has no "array" and in case the "array" is
+    # not a numpy array:
+    except AttributeError:
+        pass
+    # Mask is enforced to be a numpy.array across astropy versions
     if ccd.mask is not None:
         size_of_an_img += ccd.mask.nbytes
-    if ccd.flags is not None:
+    # flags is not necessarily a numpy array so do not fail with an
+    # AttributeError in case something was set!
+    # TODO: Flags are not taken into account in Combiner. This number is added
+    #       nevertheless for future compatibility.
+    try:
         size_of_an_img += ccd.flags.nbytes
+    except AttributeError:
+        pass
 
     no_of_img = len(img_list)
 
     # determine the number of chunks to split the images into
-    no_chunks = int((size_of_an_img*no_of_img)/mem_limit)+1
-    log.info('splitting each image into {0} chunks to limit memory usage to '
-             '{1} bytes.'.format(no_chunks, mem_limit))
+    no_chunks = int((size_of_an_img * no_of_img) / mem_limit) + 1
+    if no_chunks > 1:
+        log.info('splitting each image into {0} chunks to limit memory usage '
+                 'to {1} bytes.'.format(no_chunks, mem_limit))
     xs, ys = ccd.data.shape
     # First we try to split only along fast x axis
     xstep = max(1, int(xs/no_chunks))
@@ -615,7 +659,7 @@ def combine(img_list, output_file=None, method='average', weights=None,
 
     if clip_extrema:
         to_call_in_combiner['clip_extrema'] = {'nlow': nlow,
-                                                       'nhigh': nhigh}
+                                               'nhigh': nhigh}
 
     if minmax_clip:
         to_call_in_combiner['minmax_clipping'] = {'min_clip': minmax_clip_min,
@@ -641,10 +685,10 @@ def combine(img_list, output_file=None, method='average', weights=None,
                     imgccd = CCDData.read(image, **ccdkwargs)
 
                 # Trim image
-                ccd_list.append(trim_image(imgccd[x:xend, y:yend]))
+                ccd_list.append(imgccd[x:xend, y:yend])
 
             # Create Combiner for tile
-            tile_combiner = Combiner(ccd_list)
+            tile_combiner = Combiner(ccd_list, dtype=dtype)
             # Set all properties and call all methods
             for to_set in to_set_in_combiner:
                 setattr(tile_combiner, to_set, to_set_in_combiner[to_set])
@@ -652,7 +696,11 @@ def combine(img_list, output_file=None, method='average', weights=None,
                 getattr(tile_combiner, to_call)(**to_call_in_combiner[to_call])
 
             # Finally call the combine algorithm
-            comb_tile = getattr(tile_combiner, combine_function)()
+            combine_kwds = {}
+            if combine_uncertainty_function is not None:
+                combine_kwds['uncertainty_func'] = combine_uncertainty_function
+
+            comb_tile = getattr(tile_combiner, combine_function)(**combine_kwds)
 
             # add it back into the master image
             ccd.data[x:xend, y:yend] = comb_tile.data
