@@ -6,13 +6,16 @@ from __future__ import (absolute_import, division, print_function,
 import numbers
 
 import numpy as np
+import math
 from astropy.extern import six
 from astropy.units.quantity import Quantity
 from astropy import units as u
 from astropy.modeling import fitting
 from astropy import stats
+from astropy.nddata import utils as nddata_utils
 from astropy.nddata import StdDevUncertainty
 from astropy.wcs.utils import proj_plane_pixel_area
+from astropy.utils import deprecated
 import astropy  # To get the version.
 
 from scipy import ndimage
@@ -25,7 +28,8 @@ __all__ = ['background_deviation_box', 'background_deviation_filter',
            'ccd_process', 'cosmicray_median', 'cosmicray_lacosmic',
            'create_deviation', 'flat_correct', 'gain_correct', 'rebin',
            'sigma_func', 'subtract_bias', 'subtract_dark', 'subtract_overscan',
-           'transform_image', 'trim_image', 'wcs_project', 'Keyword']
+           'transform_image', 'trim_image', 'wcs_project', 'Keyword',
+           'median_filter', 'ccdmask']
 
 # The dictionary below is used to translate actual function names to names
 # that are FITS compliant, i.e. 8 characters or less.
@@ -228,12 +232,12 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
 
     # subtracting the master bias
     if isinstance(master_bias, CCDData):
-        nccd = nccd.subtract(master_bias)
+        nccd = subtract_bias(nccd, master_bias)
     elif master_bias is None:
         pass
     else:
         raise TypeError(
-            'master_bias is not None, numpy.ndarray, or a CCDData object.')
+            'master_bias is not None or a CCDData object.')
 
     # subtract the dark frame
     if isinstance(dark_frame, CCDData):
@@ -255,7 +259,7 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         pass
     else:
         raise TypeError(
-            'master_flat is not None, numpy.ndarray, or a CCDData object.')
+            'master_flat is not None or a CCDData object.')
 
     return nccd
 
@@ -700,6 +704,7 @@ def gain_correct(ccd, gain, gain_unit=None):
         gain_value = gain
 
     result = ccd.multiply(gain_value)
+    result.meta = ccd.meta.copy()
     return result
 
 
@@ -712,7 +717,7 @@ def flat_correct(ccd, flat, min_value=None):
     Parameters
     ----------
     ccd : `~ccdproc.CCDData`
-        Data to be flatfield corrected.
+        Data to be transformed.
 
     flat : `~ccdproc.CCDData`
         Flatfield to apply to the data.
@@ -760,8 +765,8 @@ def transform_image(ccd, transform_func, **kwargs):
     ccd : `~ccdproc.CCDData`
         Data to be flatfield corrected.
 
-    transform_func : function
-        Function to be used to transform the data.
+    transform_func : callable
+        Function to be used to transform the data, mask and uncertainty.
 
     kwargs :
         Additional keyword arguments to be used by the transform_func.
@@ -798,18 +803,19 @@ def transform_image(ccd, transform_func, **kwargs):
         >>> transformed = transform_image(arr1, shift, shift=(5.5, 8.1))
     """
     # check that it is a ccddata object
-    if not (isinstance(ccd, CCDData)):
+    if not isinstance(ccd, CCDData):
         raise TypeError('ccd is not a CCDData.')
 
-    # check that transform is a callable function
-    if not hasattr(transform_func, '__call__'):
-        raise TypeError('transform is not a function.')
-
     # make a copy of the object
     nccd = ccd.copy()
 
     # transform the image plane
-    nccd.data = transform_func(nccd.data, **kwargs)
+    try:
+        nccd.data = transform_func(nccd.data, **kwargs)
+    except TypeError as exc:
+        if 'is not callable' in str(exc):
+            raise TypeError('transform_func is not a callable.')
+        raise
 
     # transform the uncertainty plane if it exists
     if nccd.uncertainty is not None:
@@ -819,7 +825,7 @@ def transform_image(ccd, transform_func, **kwargs):
     # transform the mask plane
     if nccd.mask is not None:
         mask = transform_func(nccd.mask, **kwargs)
-        nccd.mask = (mask > 0)
+        nccd.mask = mask > 0
 
     return nccd
 
@@ -1051,6 +1057,7 @@ def background_deviation_filter(data, bbox):
     return ndimage.generic_filter(data, sigma_func, size=(bbox, bbox))
 
 
+@deprecated('1.1')
 def rebin(ccd, newshape):
     """
     Rebin an array to have a new shape.
@@ -1137,6 +1144,45 @@ def rebin(ccd, newshape):
         raise TypeError('ccd is not an ndarray or a CCDData object.')
 
 
+def block_reduce(ccd, block_size, func=np.sum):
+    """Thin wrapper around `astropy.nddata.block_reduce`."""
+    data = nddata_utils.block_reduce(ccd, block_size, func)
+    if isinstance(ccd, CCDData):
+        # unit and meta "should" be unaffected by the change of shape and can
+        # be copied. However wcs, mask, uncertainty should not be copied!
+        data = CCDData(data, unit=ccd.unit, meta=ccd.meta.copy())
+    return data
+
+
+def block_average(ccd, block_size):
+    """Like `block_reduce` but with predefined ``func=np.mean``.
+    """
+    data = nddata_utils.block_reduce(ccd, block_size, np.mean)
+    # Like in block_reduce:
+    if isinstance(ccd, CCDData):
+        data = CCDData(data, unit=ccd.unit, meta=ccd.meta.copy())
+    return data
+
+
+def block_replicate(ccd, block_size, conserve_sum=True):
+    """Thin wrapper around `astropy.nddata.block_replicate`."""
+    data = nddata_utils.block_replicate(ccd, block_size, conserve_sum)
+    # Like in block_reduce:
+    if isinstance(ccd, CCDData):
+        data = CCDData(data, unit=ccd.unit, meta=ccd.meta.copy())
+    return data
+
+
+try:
+    # Append original docstring to docstrings of these functions
+    block_reduce.__doc__ += nddata_utils.block_reduce.__doc__
+    block_replicate.__doc__ += nddata_utils.block_replicate.__doc__
+    __all__ += ['block_average', 'block_reduce', 'block_replicate']
+except AttributeError:
+    # Astropy 1.0 has no block_reduce, block_average
+    del block_reduce, block_average, block_replicate
+
+
 def _blkavg(data, newshape):
     """
     Block average an array such that it has the new shape.
@@ -1187,6 +1233,22 @@ def _blkavg(data, newshape):
     return eval(''.join(evList))
 
 
+def median_filter(data, *args, **kwargs):
+    """See `scipy.ndimage.median_filter` for arguments.
+
+    If the ``data`` is a `~ccdproc.CCDData` object the result will be another
+    `~ccdproc.CCDData` object with the median filtered data as ``data`` and
+    copied ``unit`` and ``meta``.
+    """
+    if isinstance(data, CCDData):
+        out_kwargs = {'meta': data.meta.copy(),
+                      'unit': data.unit}
+        result = ndimage.median_filter(data.data, *args, **kwargs)
+        return CCDData(result, **out_kwargs)
+    else:
+        return ndimage.median_filter(data, *args, **kwargs)
+
+
 def cosmicray_lacosmic(ccd, sigclip=4.5, sigfrac=0.3,
                        objlim=5.0, gain=1.0, readnoise=6.5,
                        satlevel=65535.0, pssl=0.0, niter=4,
@@ -1515,6 +1577,169 @@ def cosmicray_median(ccd, error_image=None, thresh=5, mbox=11, gbox=0,
         raise TypeError('ccd is not an numpy.ndarray or a CCDData object.')
 
 
+def ccdmask(ratio, findbadcolumns=False, byblocks=False, ncmed=7, nlmed=7,
+            ncsig=15, nlsig=15, lsigma=9, hsigma=9, ngood=5):
+    """
+    Uses method based on the IRAF ccdmask task to generate a mask based on the
+    given input.
+
+    .. note::
+        This function uses ``lines`` as synonym for the first axis and
+        ``columns`` the second axis. Only two-dimensional ``ratio`` is
+        currently supported.
+
+    Parameters
+    ----------
+    ratio : `~ccdproc.CCDData`
+        Data to used to form mask.  Typically this is the ratio of two flat
+        field images.
+
+    findbadcolumns: `bool`, optional
+        If set to True, the code will search for bad column sections.  Note
+        that this treats columns as special and breaks symmetry between lines
+        and columns and so is likely only appropriate for detectors which have
+        readout directions.
+
+    byblocks : `bool`, optional
+        If set to true, the code will divide the image up in to blocks of size
+        nlsig by ncsig and determine the standard deviation estimate in each
+        block (as described in the original IRAF task, see Notes below).  If
+        set to False, then the code will use `scipy.ndimage.percentile_filter`
+        to generate a running box version of the standard
+        deviation estimate and use that value for the standard deviation at
+        each pixel.
+
+    ncmed, nlmed: `int`, optional
+        The column and line size of the moving median rectangle used to
+        estimate the uncontaminated local signal. The column median size should
+        be at least 3 pixels to span single bad columns.
+
+    ncsig, nlsig: `int`, optional
+        The column and line size of regions used to estimate the uncontaminated
+        local sigma using a percentile. The size of the box should contain of
+        order 100 pixels or more.
+
+    lsigma, hsigma: `float`, optional
+        Positive sigma factors to use for selecting pixels below and above the
+        median level based on the local percentile sigma.
+
+    ngood: `int`, optional
+        Gaps of undetected pixels along the column direction of length less
+        than this amount are also flagged as bad pixels, if they are between
+        pixels masked in that column.
+
+    Returns
+    -------
+    mask : `numpy.ndarray`
+        A boolean ndarray where the bad pixels have a value of 1 (True) and
+        valid pixels 0 (False), following the numpy.ma conventions.
+
+    Notes
+    -----
+    Similar implementation to IRAF's ccdmask task.
+    The Following documentation is copied directly from:
+    http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?ccdmask
+
+    The input image is first subtracted by a moving box median. The median is
+    unaffected by bad pixels provided the median size is larger that twice the
+    size of a bad region. Thus, if 3 pixel wide bad columns are present then
+    the column median box size should be at least 7 pixels. The median box can
+    be a single pixel wide along one dimension if needed. This may be
+    appropriate for spectroscopic long slit data.
+
+    The median subtracted image is then divided into blocks of size nclsig by
+    nlsig. In each block the pixel values are sorted and the pixels nearest the
+    30.9 and 69.1 percentile points are found; this would be the one sigma
+    points in a Gaussian noise distribution. The difference between the two
+    count levels divided by two is then the local sigma estimate. This
+    algorithm is used to avoid contamination by the bad pixel values. The block
+    size must be at least 10 pixels in each dimension to provide sufficient
+    pixels for a good estimate of the percentile sigma. The sigma uncertainty
+    estimate of each pixel in the image is then the sigma from the nearest
+    block.
+
+    The deviant pixels are found by comparing the median subtracted residual to
+    a specified sigma threshold factor times the local sigma above and below
+    zero (the lsigma and hsigma parameters). This is done for individual pixels
+    and then for column sums of pixels (excluding previously flagged bad
+    pixels) from two to the number of lines in the image. The sigma of the sums
+    is scaled by the square root of the number of pixels summed so that
+    statistically low or high column regions may be detected even though
+    individual pixels may not be statistically deviant. For the purpose of this
+    task one would normally select large sigma threshold factors such as six or
+    greater to detect only true bad pixels and not the extremes of the noise
+    distribution.
+
+    As a final step each column is examined to see if there are small segments
+    of unflagged pixels between bad pixels. If the length of a segment is less
+    than that given by the ngood parameter all the pixels in the segment are
+    also marked as bad.
+    """
+    try:
+        nlines, ncols = ratio.data.shape
+    except (TypeError, ValueError):
+        # shape is not iterable or has more or less than two values
+        raise ValueError('"ratio" must be two-dimensional.')
+    except AttributeError:
+        # No data attribute or data has no shape attribute.
+        raise ValueError('"ratio" should be a "CCDData".')
+
+    def _sigma_mask(baseline, one_sigma_value, lower_sigma, upper_sigma):
+        """Helper function to mask values outside of the specified sigma range.
+        """
+        return ((baseline < -lower_sigma * one_sigma_value) |
+                (baseline > upper_sigma * one_sigma_value))
+
+    mask = ~np.isfinite(ratio.data)
+    medsub = (ratio.data -
+              ndimage.median_filter(ratio.data, size=(nlmed, ncmed)))
+
+    if byblocks:
+        nlinesblock = int(math.ceil(nlines / nlsig))
+        ncolsblock = int(math.ceil(ncols / ncsig))
+        for i in six.moves.range(nlinesblock):
+            for j in six.moves.range(ncolsblock):
+                l1 = i * nlsig
+                l2 = min((i + 1) * nlsig, nlines)
+                c1 = j * ncsig
+                c2 = min((j + 1) * ncsig, ncols)
+                block = medsub[l1:l2, c1:c2]
+                high = np.percentile(block.ravel(), 69.1)
+                low = np.percentile(block.ravel(), 30.9)
+                block_sigma = (high - low) / 2.0
+                block_mask = _sigma_mask(block, block_sigma, lsigma, hsigma)
+                mblock = np.ma.MaskedArray(block, mask=block_mask, copy=False)
+
+                if findbadcolumns:
+                    csum = np.ma.sum(mblock, axis=0)
+                    csum[csum <= 0] = 0
+                    csum_sigma = np.ma.MaskedArray(np.sqrt(c2 - c1 - csum))
+                    colmask = _sigma_mask(csum.filled(1), csum_sigma,
+                                          lsigma, hsigma)
+                    block_mask[:, :] |= colmask[np.newaxis, :]
+
+                mask[l1:l2, c1:c2] = block_mask
+    else:
+        high = ndimage.percentile_filter(medsub, 69.1, size=(nlsig, ncsig))
+        low = ndimage.percentile_filter(medsub, 30.9, size=(nlsig, ncsig))
+        sigmas = (high - low) / 2.0
+        mask |= _sigma_mask(medsub, sigmas, lsigma, hsigma)
+
+    if findbadcolumns:
+        # Loop through columns and look for short segments (<ngood pixels long)
+        # which are unmasked, but are surrounded by masked pixels and mask them
+        # under the assumption that the column region is bad.
+        for col in six.moves.range(0, ncols):
+            for line in six.moves.range(0, nlines - ngood - 1):
+                if mask[line, col]:
+                    for i in six.moves.range(2, ngood + 2):
+                        lend = line + i
+                        if (mask[lend, col] and
+                                not np.all(mask[line:lend + 1, col])):
+                            mask[line:lend, col] = True
+    return mask
+
+
 class Keyword(object):
     """
     """
