@@ -15,6 +15,8 @@ from astropy.extern import six
 import warnings
 from astropy.utils.exceptions import AstropyUserWarning
 
+from .ccddata import fits_ccddata_reader
+
 logger = logging.getLogger(__name__)
 
 __all__ = ['ImageFileCollection']
@@ -34,7 +36,7 @@ class ImageFileCollection(object):
     Parameters
     ----------
     location : str or None, optional
-        path to directory containing FITS files.
+        Path to directory containing FITS files.
         Default is ``None``.
 
     keywords : list of str, '*' or None, optional
@@ -51,17 +53,25 @@ class ImageFileCollection(object):
         list.
         Default is ``None``.
 
+    filenames: str, list of str, or None, optional
+        List of the names of FITS files which will be added to the collection.
+        The filenames are assumed to be in ``location``.
+        Default is ``None``.
+
     Raises
     ------
     ValueError
         Raised if keywords are set to a combination of '*' and any other
         value.
     """
-    def __init__(self, location=None, keywords=None, info_file=None):
+    def __init__(self, location=None, keywords=None, info_file=None,
+                 filenames=None):
         self._location = location
+        self._filenames = filenames
         self._files = []
         if location:
-            self._files = self._fits_files_in_directory()
+            self._files = self._get_files()
+
         if self._files == []:
             warnings.warn("no FITS files in the collection.",
                           AstropyUserWarning)
@@ -262,7 +272,8 @@ class ImageFileCollection(object):
         filtered_files = self.summary_info['file'].compressed()
         self.summary_info['file'].mask = current_file_mask
         if include_path:
-            filtered_files = [path.join(self._location, f) for f in filtered_files]
+            filtered_files = [path.join(self._location, f)
+                              for f in filtered_files]
         return filtered_files
 
     def refresh(self):
@@ -271,7 +282,7 @@ class ImageFileCollection(object):
         """
         keywords = '*' if self._all_keywords else self.keywords
         # Re-load list of files
-        self._files = self._fits_files_in_directory()
+        self._files = self._get_files()
         self._summary_info = self._fits_summary(header_keywords=keywords)
 
     def sort(self, keys=None):
@@ -290,6 +301,26 @@ class ImageFileCollection(object):
             self._summary_info.sort(keys)
             self._files = list(self.summary_info['file'])
 
+    def _get_files(self):
+        """ Helper method which checks whether ``files`` should be set
+        to a subset of file names or to all file names in a directory.
+
+        Returns
+        -------
+        files : list or str
+            List of file names which will be added to the collection.
+        """
+        files = []
+        if self._filenames:
+            if isinstance(self._filenames, six.string_types):
+                files.append(self._filenames)
+            else:
+                files = self._filenames
+        else:
+            files = self._fits_files_in_directory()
+
+        return files
+
     def _dict_from_fits_header(self, file_name, input_summary=None,
                                missing_marker=None):
         """
@@ -573,6 +604,7 @@ class ImageFileCollection(object):
                    overwrite=False,
                    do_not_scale_image_data=True,
                    return_fname=False,
+                   ccd_kwargs=None,
                    **kwd):
         """
         Generator that yields each {name} in the collection.
@@ -619,6 +651,14 @@ class ImageFileCollection(object):
             not the full path to the file.
             Default is ``False``.
 
+        ccd_kwargs : dict, optional
+            Dict with parameters for `~ccdproc.fits_ccddata_reader`.
+            For instance, the key ``'unit'`` can be used to specify the unit
+            of the data. If ``'unit'`` is not given then ``'adu'`` is used as
+            the default unit.
+            See `~ccdproc.fits_ccddata_reader` for a complete list of
+            parameters that can be passed through ``ccd_kwargs``.
+
         kwd :
             Any additional keywords are used to filter the items returned; see
             Examples for details.
@@ -645,6 +685,8 @@ class ImageFileCollection(object):
         if kwd:
             self._find_keywords_by_values(**kwd)
 
+        ccd_kwargs = ccd_kwargs or {}
+
         for full_path in self._paths():
             no_scale = do_not_scale_image_data
             hdulist = fits.open(full_path,
@@ -652,14 +694,16 @@ class ImageFileCollection(object):
 
             file_name = path.basename(full_path)
 
-            return_options = {'header': hdulist[0].header,
-                              'hdu': hdulist[0],
-                              'data': hdulist[0].data}
-
+            return_options = {
+                    'header': lambda: hdulist[0].header,
+                    'hdu': lambda: hdulist[0],
+                    'data': lambda: hdulist[0].data,
+                    'ccd': lambda: fits_ccddata_reader(full_path, **ccd_kwargs)
+                    }
             try:
-                yield (return_options[return_type]  # pragma: no branch
+                yield (return_options[return_type]()  # pragma: no branch
                        if (not return_fname) else
-                       (return_options[return_type], file_name))
+                       (return_options[return_type](), file_name))
             except KeyError:
                 raise ValueError('no generator for {}'.format(return_type))
 
@@ -701,22 +745,25 @@ class ImageFileCollection(object):
         return self._generator('header',
                                do_not_scale_image_data=do_not_scale_image_data,
                                **kwd)
-    headers.__doc__ = _generator.__doc__.format(name='header',
-                                                default_scaling='True',
-                                                return_type='astropy.io.fits.Header')
+    headers.__doc__ = _generator.__doc__.format(
+        name='header', default_scaling='True',
+        return_type='astropy.io.fits.Header')
 
     def hdus(self, do_not_scale_image_data=False, **kwd):
         return self._generator('hdu',
                                do_not_scale_image_data=do_not_scale_image_data,
                                **kwd)
-    hdus.__doc__ = _generator.__doc__.format(name='HDU',
-                                             default_scaling='False',
-                                             return_type='astropy.io.fits.HDU')
+    hdus.__doc__ = _generator.__doc__.format(
+        name='HDU', default_scaling='False', return_type='astropy.io.fits.HDU')
 
     def data(self, do_not_scale_image_data=False, **kwd):
         return self._generator('data',
                                do_not_scale_image_data=do_not_scale_image_data,
                                **kwd)
-    data.__doc__ = _generator.__doc__.format(name='image',
-                                             default_scaling='False',
-                                             return_type='numpy.ndarray')
+    data.__doc__ = _generator.__doc__.format(
+        name='image', default_scaling='False', return_type='numpy.ndarray')
+
+    def ccds(self, ccd_kwargs=None, **kwd):
+        return self._generator('ccd', ccd_kwargs=ccd_kwargs, **kwd)
+    ccds.__doc__ = _generator.__doc__.format(
+        name='CCDData', default_scaling='True', return_type='ccdproc.CCDData')
