@@ -19,6 +19,13 @@ from ..ccddata import CCDData
 from ..core import *
 from ..core import _blkavg
 
+try:
+    from ..core import block_reduce
+    HAS_BLOCK_X_FUNCS = True
+except:
+    HAS_BLOCK_X_FUNCS = False
+
+
 
 # test creating deviation
 # success expected if u_image * u_gain = u_readnoise
@@ -612,6 +619,77 @@ def test_rebin_ccddata(ccd_data, mask_data, uncertainty):
         assert b.uncertainty.array.shape == (20, 20)
 
 
+# test block_reduce and block_replicate wrapper
+@pytest.mark.skipif(not HAS_BLOCK_X_FUNCS, reason="needs astropy >= 1.1.x")
+def test_block_reduce():
+    ccd = CCDData(np.ones((4, 4)), unit='adu', meta={'testkw': 1},
+                  mask=np.zeros((4, 4), dtype=bool),
+                  uncertainty=StdDevUncertainty(np.ones((4, 4))),
+                  wcs=np.zeros((4, 4)))
+    ccd_summed = block_reduce(ccd, (2, 2))
+    assert isinstance(ccd_summed, CCDData)
+    assert np.all(ccd_summed.data == 4)
+    assert ccd_summed.data.shape == (2, 2)
+    assert ccd_summed.unit == u.adu
+    # Other attributes are set to None. In case the function is modified to
+    # work on these attributes correctly those tests need to be updated!
+    assert ccd_summed.meta == {'testkw': 1}
+    assert ccd_summed.mask is None
+    assert ccd_summed.wcs is None
+    assert ccd_summed.uncertainty is None
+
+    # Make sure meta is copied
+    ccd_summed.meta['testkw2'] = 10
+    assert 'testkw2' not in ccd.meta
+
+
+@pytest.mark.skipif(not HAS_BLOCK_X_FUNCS, reason="needs astropy >= 1.1.x")
+def test_block_average():
+    ccd = CCDData(np.ones((4, 4)), unit='adu', meta={'testkw': 1},
+                  mask=np.zeros((4, 4), dtype=bool),
+                  uncertainty=StdDevUncertainty(np.ones((4, 4))),
+                  wcs=np.zeros((4, 4)))
+    ccd.data[::2, ::2] = 2
+    ccd_avgd = block_average(ccd, (2, 2))
+    assert isinstance(ccd_avgd, CCDData)
+    assert np.all(ccd_avgd.data == 1.25)
+    assert ccd_avgd.data.shape == (2, 2)
+    assert ccd_avgd.unit == u.adu
+    # Other attributes are set to None. In case the function is modified to
+    # work on these attributes correctly those tests need to be updated!
+    assert ccd_avgd.meta == {'testkw': 1}
+    assert ccd_avgd.mask is None
+    assert ccd_avgd.wcs is None
+    assert ccd_avgd.uncertainty is None
+
+    # Make sure meta is copied
+    ccd_avgd.meta['testkw2'] = 10
+    assert 'testkw2' not in ccd.meta
+
+
+@pytest.mark.skipif(not HAS_BLOCK_X_FUNCS, reason="needs astropy >= 1.1.x")
+def test_block_replicate():
+    ccd = CCDData(np.ones((4, 4)), unit='adu', meta={'testkw': 1},
+                  mask=np.zeros((4, 4), dtype=bool),
+                  uncertainty=StdDevUncertainty(np.ones((4, 4))),
+                  wcs=np.zeros((4, 4)))
+    ccd_repl = block_replicate(ccd, (2, 2))
+    assert isinstance(ccd_repl, CCDData)
+    assert np.all(ccd_repl.data == 0.25)
+    assert ccd_repl.data.shape == (8, 8)
+    assert ccd_repl.unit == u.adu
+    # Other attributes are set to None. In case the function is modified to
+    # work on these attributes correctly those tests need to be updated!
+    assert ccd_repl.meta == {'testkw': 1}
+    assert ccd_repl.mask is None
+    assert ccd_repl.wcs is None
+    assert ccd_repl.uncertainty is None
+
+    # Make sure meta is copied
+    ccd_repl.meta['testkw2'] = 10
+    assert 'testkw2' not in ccd.meta
+
+
 #test blockaveraging ndarray
 def test__blkavg_ndarray(ccd_data):
     with pytest.raises(TypeError):
@@ -809,10 +887,8 @@ def test_wcs_project_onto_scale_wcs(ccd_data):
     assert new_ccd.wcs.wcs.compare(target_wcs.wcs)
 
     # Define a cutout from the new array that should match the old.
-    new_lower_bound = (np.array(new_ccd.shape) -
-                       np.array(ccd_data.shape)) / 2
-    new_upper_bound = (np.array(new_ccd.shape) +
-                       np.array(ccd_data.shape)) / 2
+    new_lower_bound = (np.array(new_ccd.shape) - np.array(ccd_data.shape)) // 2
+    new_upper_bound = (np.array(new_ccd.shape) + np.array(ccd_data.shape)) // 2
     data_cutout = new_ccd.data[new_lower_bound[0]:new_upper_bound[0],
                                new_lower_bound[1]:new_upper_bound[1]]
 
@@ -824,7 +900,7 @@ def test_wcs_project_onto_scale_wcs(ccd_data):
 
     # Mask should be true for four pixels (all nearest neighbors)
     # of the single pixel we masked initially.
-    new_center = new_ccd.wcs.wcs.crpix
+    new_center = np.array(new_ccd.wcs.wcs.crpix, dtype=int, copy=False)
     assert np.all(new_ccd.mask[new_center[0]:new_center[0]+2,
                                new_center[1]:new_center[1]+2])
 
@@ -877,6 +953,7 @@ def test_ccd_process():
     # test the through ccd_process
     ccd_data = CCDData(10.0 * np.ones((100, 100)), unit=u.adu)
     ccd_data.data[:, -10:] = 2
+    ccd_data.meta['testkw'] = 100
 
     mask = np.zeros((100, 90))
 
@@ -905,3 +982,5 @@ def test_ccd_process():
                                   occd.uncertainty.array)
     np.testing.assert_array_equal(mask, occd.mask)
     assert(occd.unit == u.electron)
+    # Make sure the original keyword is still present. Regression test for #401
+    assert occd.meta['testkw'] == 100
