@@ -14,6 +14,8 @@ import numpy as np
 from astropy.tests.helper import pytest, catch_warnings
 from astropy.utils.exceptions import AstropyUserWarning
 
+from ccdproc import CCDData
+
 from .. import image_collection
 
 _filters = []
@@ -85,6 +87,20 @@ class TestImageFileCollection(object):
             location=triage_setup.test_dir, keywords=['imagetyp', 'filter'])
         assert img_collection.summary is img_collection.summary_info
 
+    def test_filenames_are_set_properly(self, triage_setup):
+        fn = ['filter_no_object_bias.fit', 'filter_object_light_foo.fit']
+        img_collection = image_collection.ImageFileCollection(
+            location=triage_setup.test_dir, filenames=fn, keywords=['filter'])
+        assert img_collection.files == fn
+
+        img_collection.refresh()
+        assert img_collection.files == fn
+
+        fn = 'filter_no_object_bias.fit'
+        img_collection = image_collection.ImageFileCollection(
+            location=triage_setup.test_dir, filenames=fn, keywords=['filter'])
+        assert img_collection.files == [fn]
+
     def test_files_with_compressed(self, triage_setup):
         collection = image_collection.ImageFileCollection(
             location=triage_setup.test_dir)
@@ -217,6 +233,20 @@ class TestImageFileCollection(object):
         for img in collection.data():
             assert isinstance(img, np.ndarray)
 
+    def test_generator_ccds_without_unit(self, triage_setup):
+        collection = image_collection.ImageFileCollection(
+                location=triage_setup.test_dir, keywords=['imagetyp'])
+
+        with pytest.raises(ValueError):
+            ccd = next(collection.ccds())
+
+    def test_generator_ccds(self, triage_setup):
+        collection = image_collection.ImageFileCollection(
+                location=triage_setup.test_dir, keywords=['imagetyp'])
+        ccd_kwargs = {'unit': 'adu'}
+        for ccd in collection.ccds(ccd_kwargs=ccd_kwargs):
+            assert isinstance(ccd, CCDData)
+
     def test_consecutive_fiilters(self, triage_setup):
         collection = image_collection.ImageFileCollection(location=triage_setup.test_dir,
                                              keywords=['imagetyp',
@@ -627,3 +657,23 @@ class TestImageFileCollection(object):
             assert((str(hdu.header), fname) in all_elements)
         for i in range(len(collection.summary)):
             assert(collection.summary['file'][i] == collection.files[i])
+
+    def test_ccds_generator_in_different_directory(self, triage_setup, tmpdir):
+        """
+        Regression test for https://github.com/astropy/ccdproc/issues/421 in
+        which the ccds generator fails if the current working directory is
+        not the location of the ImageFileCollection.
+        """
+
+        coll = image_collection.ImageFileCollection(triage_setup.test_dir)
+
+        # The temporary directory below should be different that the collection
+        # location.
+        os.chdir(tmpdir.strpath)
+
+        # Let's make sure it is.
+        assert not os.path.samefile(os.getcwd(), coll.location)
+
+        # This generated an IOError before the issue was fixed.
+        for _ in coll.ccds(ccd_kwargs={'unit': 'adu'}):
+            pass
